library volume3d;

{
Volume3D
3D function library for CAM purposes

program author
Arndt Kritzner, see http://www.kritzner.de

version
1.0.5 ... 2005-03-13

licenses:
volume3d is available under the terms of GNUs GPL (for developers of free software,
only FreePascal or GNU Pascal with Linux) or alternatively under a commercial
license (for developers of commercial software)

All rights reserved.
Any usage in another kind than described in the purchased license is prohibited.
If you want to choose the GNU GPL on another platform than GNU/Linux ask for a
free license for your preferred OS mailto:arndt@kritzner.de.

See http://www.kritzner.de/pages/v3d/v3d.html for technical and licensing details.
}

{
Library volume3d
volume3d library export functions for usage as dynamic library
}

{$ifndef WIN32}
{$define CDECL}
{$else}
{$define STDCALL}
{$endif}

uses triangle3d,shape3d,form3d,trans3d,export3d;

{================================================================================================}
{basic data type and structure info and access functions}
function v3_call_version:string;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_call_version:={$ifdef STDCALL}'STDCALL'{$else}{$ifdef CDECL}'CDECL'{$else}'?'{$endif}{$endif}; end;

function v3_size_tri_idx3d:word;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_size_tri_idx3d:=sizeof(tri_idx3d); end;

function v3_size_float3d:word;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_size_float3d:=sizeof(float3d); end;

function v3_size_poly_idx3d:word;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_size_poly_idx3d:=sizeof(poly_idx3d); end;

function v3_size_tpoint:word;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_size_tpoint:=sizeof(tpoint); end;

function v3_size_triangle:word;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_size_triangle:=sizeof(ttriangle); end;

procedure v3_info;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin
  WriteLN('volume3d dynamic library v1.0.5');
  WriteLN('calling scheme: ',v3_call_version);
  Writeln('data types:');
  Writeln('  tri_idx3d : ',v3_size_tri_idx3d,' bytes');
  Writeln('  float3d   : ',v3_size_float3d,' bytes');
  Writeln('  poly_idx3d: ',v3_size_poly_idx3d,' bytes');
  Writeln('  tpoint    : ',v3_size_tpoint,' bytes');
  Writeln('  ttriangle : ',v3_size_triangle,' bytes');
end;

{================================================================================================}
{functions from triangle3d}
procedure v3_minimize_list(shape:pointer);
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin pshape_3d(shape)^.change_size(cl_minimize); end;

procedure v3_maximize_list(shape:pointer);
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin pshape_3d(shape)^.change_size(cl_maximize); end;

function v3_triangles_count(shape:pointer):tri_idx3d;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_triangles_count:=pshape_3d(shape)^.count; end;

function v3_ins_triangle(shape:pointer;t:pointer):boolean;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_ins_triangle:=pshape_3d(shape)^.ins_triangle(t); end;

function v3_ins_triangle_idx(shape:pointer;idx:tri_idx3d;t:pointer):boolean;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_ins_triangle_idx:=pshape_3d(shape)^.ins_triangle(idx,t); end;

function v3_del_triangle(shape:pointer;idx:tri_idx3d):boolean;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_del_triangle:=pshape_3d(shape)^.del_triangle(idx); end;

function v3_get_triangle(shape:pointer;idx:tri_idx3d):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_get_triangle:=pshape_3d(shape)^.get_triangle(idx); end;

function v3_get_point(triangle:pointer;idx:byte):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin
  case idx of
    1: v3_get_point:=@(ttriangle(triangle^).p1);
    2: v3_get_point:=@(ttriangle(triangle^).p2);
    3: v3_get_point:=@(ttriangle(triangle^).p3);
  end;
end;

procedure v3_set_point(triangle:pointer;idx:byte;x,y,z:float3d);
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
var p:^tpoint;
begin
  p:=v3_get_point(triangle,idx);
  p^.x:=x;
  p^.y:=y;
  p^.z:=z;
end;

procedure v3_list_triangles(shape:pointer);
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin pshape_3d(shape)^.list_triangles(); end;

function v3_clone_shape(shape:pointer;nam:pointer):pshape_3d;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_clone_shape:=pshape_3d(shape)^.clone_shape(strpas(nam)); end;

function v3_include_triangles(shape:pointer;s:pointer):boolean;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_include_triangles:=pshape_3d(shape)^.include_triangles(s); end;

function v3_sum_area(shape:pointer):float3d;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_sum_area:=pshape_3d(shape)^.sum_area; end;

procedure v3_dispose_shape(shape:pointer);
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
var s:pshape_3d;
begin
  if assigned(shape) then begin
    s:=shape;
    dispose(s,done);
  end;
end;

{================================================================================================}
{functions from shape3d}
procedure v3_inside2out(s:pointer);
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin inside2out(s); end;

procedure v3_facet_orientation(s:pointer);
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin facet_orientation(s); end;

function v3_add_shape(s1,s2:pointer):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_add_shape:=add_shape(s1,s2); end;

function v3_sub_shape(s1,s2:pointer):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_sub_shape:=sub_shape(s1,s2); end;

function v3_inter_shape(s1,s2:pointer):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_inter_shape:=inter_shape(s1,s2); end;

{================================================================================================}
{functions from form3d}
function v3_empty(name:pointer;max_:tri_idx3d):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
var s:pshape_3d;
begin new(s,init(max_,strpas(name))); v3_empty:=s; end;

function v3_triangle_xyz(x1,y1,z1,x2,y2,z2,x3,y3,z3:float3d;name:pointer;max_:tri_idx3d):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_triangle_xyz:=triangle(x1,y1,z1,x2,y2,z2,x3,y3,z3,strpas(name),max_); end;

function v3_triangle_p(p1,p2,p3:tpoint;name:pointer;max_:tri_idx3d):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_triangle_p:=triangle(p1,p2,p3,strpas(name),max_); end;

function v3_box_xyz(x1,y1,z1,x2,y2,z2:float3d;name:pointer;max_:tri_idx3d):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_box_xyz:=box(x1,y1,z1,x2,y2,z2,strpas(name),max_); end;

function v3_box_p(p1,p2:tpoint;name:pointer;max_:tri_idx3d):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_box_p:=box(p1,p2,strpas(name),max_); end;

function v3_prism(z1,z2:float3d;no_points:poly_idx3d;points:pointer;name:pointer;max_:tri_idx3d):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_prism:=prism(z1,z2,no_points,points,strpas(name),max_); end;

function v3_icosaeder(r:float3d;name:pointer;max_:tri_idx3d):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_icosaeder:=icosaeder(r,strpas(name),max_); end;

function v3_triangle_sphere(s:pointer;name:pointer):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_triangle_sphere:=triangle_sphere(s,strpas(name)); end;

function v3_sphere(r:float3d;name:pointer;max_:tri_idx3d):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_sphere:=sphere(r,strpas(name),max_); end;

function v3_cone(r1,r2,h:float3d;name:pointer;max_:tri_idx3d):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_cone:=cone(r1,r2,h,strpas(name),max_); end;

function v3_cylinder(r,h:float3d;name:pointer;max_:tri_idx3d):pointer;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_cylinder:=cylinder(r,h,strpas(name),max_); end;

{================================================================================================}
{functions from trans3d}
function v3_translate(x,y,z:float3d; o:pointer):boolean;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_translate:=translate(x,y,z,o); end;

function v3_rotate(x1,y1,z1,x2,y2,z2,a:float3d; o:pointer):boolean;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_rotate:=rotate(x1,y1,z1,x2,y2,z2,a,o); end;

function v3_scale(x,y,z,s:float3d; o:pointer):boolean;
{$ifdef STDCALL}stdcall{$else}{$ifdef CDECL}cdecl{$endif}{$endif}; export;
begin v3_scale:=scale(x,y,z,s,o); end;

exports

v3_call_version,v3_size_tri_idx3d,v3_size_float3d,v3_size_poly_idx3d,v3_size_tpoint,v3_size_triangle,v3_info,

v3_minimize_list,v3_maximize_list,v3_triangles_count,v3_ins_triangle,v3_ins_triangle_idx,v3_del_triangle,v3_get_triangle,
v3_get_point,v3_set_point,v3_list_triangles,v3_clone_shape,v3_include_triangles,v3_sum_area,v3_dispose_shape,

v3_inside2out,v3_facet_orientation,v3_add_shape,v3_sub_shape,v3_inter_shape,

v3_empty,v3_triangle_xyz,v3_triangle_p,v3_box_xyz,v3_box_p,v3_prism,v3_icosaeder,v3_triangle_sphere,v3_sphere,v3_cone,
v3_cylinder,

v3_translate,v3_rotate,v3_scale
;

end.
